/*
 * Segment 8 Parametrierung und Konfiguration, SWE 8.2 Parametrierung - Bibliothek
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.puk.param.lib.daten;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Diese Klasse liefert ein {@link UrlasserInfo}-Objekt, welches die Inhalte der
 * Urlasserabfrage, bestehend aus:
 * <ul>
 * <li>Benutzername,</li>
 * <li>Passwort,</li>
 * <li>Ursache und</li>
 * <li>Veranlasser</li>
 * </ul>
 * beschreibt.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker,HofmannJ
 */
public class UrlasserInfo {

	private static final Debug LOGGER = Debug.getLogger();

	private static final String KEIN_BENUTZER_STRING = "<Kein Benutzer>";

	/** Benutzer. */
	private final SystemObject benutzer;

	/** Passwort. */
	private final String passwort;

	/** Ursache. */
	private final String ursache;

	/** Veranlasser. */
	private final String veranlasser;

	/**
	 * Konstruktor. übernimmt Benutzer, Ursache und den Veranlasser
	 * 
	 * @param benutzer
	 *            das Benutzerobjekt
	 * @param passwort
	 *            das Passwort
	 * @param ursache
	 *            die Ursache
	 * @param veranlasser
	 *            der Veranlasser
	 */
	public UrlasserInfo(final SystemObject benutzer, final String passwort, final String ursache,
			final String veranlasser) {
		this.benutzer = benutzer;
		this.passwort = passwort;
		this.veranlasser = veranlasser;
		this.ursache = ursache;
	}

	/**
	 * Liefert den Benutzer.
	 * 
	 * @return Benutzer. Kann <code>null</code> sein, wenn kein Benutzer gesetzt
	 *         wurde.
	 */
	public SystemObject getBenutzer() {
		return benutzer;
	}

	/**
	 * Liefert die Ursache oder einen Leerstring, wenn keine Ursache angegeben
	 * wurde.
	 * 
	 * @return die Ursache oder einen Leerstring
	 */
	public String getUrsache() {
		if (ursache == null) {
			return "";
		}
		return ursache;
	}

	/**
	 * Liefert den Veranlasser oder einen Leerstring, wenn keiner angegeben wurde.
	 * 
	 * @return den Veranlasser oder einen Leerstring.
	 */
	public String getVeranlasser() {
		if (veranlasser == null) {
			return "";
		}
		return veranlasser;
	}

	/**
	 * liefert den Name des Benutzers. Wenn kein Benutzer enthalten ist wird der
	 * {@value #KEIN_BENUTZER_STRING} geliefert.
	 * 
	 * @return den Name oder {@value #KEIN_BENUTZER_STRING}
	 */
	public String getBenutzerName() {
		if (benutzer == null) {
			return UrlasserInfo.KEIN_BENUTZER_STRING;
		}
		return benutzer.getNameOrPidOrId();
	}

	/**
	 * liefert das übergebene Passwort oder einen Leerstring, wenn keines übergeben
	 * wurde.
	 * 
	 * @return das Passwort oder einen Leerstring
	 */
	public String getPasswort() {
		if (passwort == null) {
			return "";
		}
		return passwort;
	}

	/**
	 * Befüllt den Datensatz mit den UrlasserInformationen.
	 * 
	 * Wenn der Datensatz keine entsprechenden Attribute enthält, wird eine
	 * entsprechende Log-Meldung ausgegeben.
	 * 
	 * @param daten
	 *            der zu befüllende Datensatz
	 */
	public void updateData(Data daten) {
		if (daten == null) {
			LOGGER.warning("Die übergebenen Daten dürfen nicht null sein!");
			return;
		}

		try {
			Data urlasserItem = daten.getItem("Urlasser");
			urlasserItem.getReferenceValue("BenutzerReferenz").setSystemObject(benutzer);
			urlasserItem.getTextValue("Ursache").setText(ursache == null ? "" : ursache);
			urlasserItem.getTextValue("Veranlasser").setText(veranlasser == null ? "" : veranlasser);
		} catch (Exception e) {
			LOGGER.warning("Kein Urlasser-Attribut ermittelbar: " + daten + ": " + e.getLocalizedMessage());
		}
	}

	/**
	 * liefert die UrlasserInformation aus dem übergebenen Datensatz.
	 * 
	 * Wenn der Datensatz keine entsprechenden Attribute enthält, wird null
	 * geliefert.
	 * 
	 * @param daten
	 *            der Datensatz
	 * @return die UrlasserInformationen oder null
	 */
	public static UrlasserInfo getUrlasserInfo(Data daten) {

		try {
			Data urlasserItem = daten.getItem("Urlasser");
			SystemObject benutzerObject = urlasserItem.getReferenceValue("BenutzerReferenz").getSystemObject();
			String ursacheText = urlasserItem.getTextValue("Ursache").getText();
			String veranlasserText = urlasserItem.getTextValue("Veranlasser").getText();

			return new UrlasserInfo(benutzerObject, null, ursacheText, veranlasserText);

		} catch (Exception e) {
			LOGGER.warning("Kein Urlasser-Attribut ermittelbar: " + daten + ": " + e.getLocalizedMessage());
		}

		return null;
	}
}
